package api

import (
	"encoding/json"
	"net/http"
	"net/url"
	"strconv"
	"strings"
	"time"

	"github.com/gin-gonic/gin"
)

const IDFM_BASEURL = "https://api-iv.iledefrance-mobilites.fr/"

type IDFMSchedule struct {
	NextDepartures struct {
		StatusCode int `json:"statusCode"`
		Data       []struct {
			LineId        string `json:"lineId"`
			ShortName     string `json:"shortName"`
			VehicleName   string `json:"vehicleName,omitempty"`
			LineDirection string `json:"lineDirection"`
			Sens          string `json:"sens,omitempty"`
			Code          string `json:"code,omitempty"`
			Time          string `json:"time"`
			Schedule      string `json:"schedule"`
			Destination   struct {
				StopPointId string `json:"stopPointId"`
				StopAreaId  string `json:"stopAreaId"`
			} `json:"destination,omitempty"`
			Source string `json:"source,omitempty"`
		} `json:"data"`
	} `json:"nextDepartures"`
	CrowdsourcingReports struct {
		congestions []struct {
			DirectionId     string `json:"directionId"`
			NearTimeReports struct {
				Rating *string `json:"rating"`
			} `json:"nearTimeReports"`
		} `json:"congestions"`
	} `json:"crowdsourcingReports"`
}

type PGSchedule struct {
	Destination string `json:"destination"`
	Message     string `json:"message"`
	Code        string `json:"code,omitempty"`
}

func convertLineCode(code string) string {
	return strings.TrimSuffix(strings.Replace(code, "STIF:Line::", "IDFM:", 1), ":")
}

func declareSchedulesRoutes(router *gin.RouterGroup) {
	router.GET("/schedules/:type/:code/:station/:way", func(c *gin.Context) {
		t := convertLineType(string(c.Param("type")))
		code := convertLineType(string(c.Param("code")))
		station := string(c.Param("station"))
		way := string(c.Param("way"))

		if !strings.HasPrefix(code, "line:IDFM:") {
			if len(code) != 6 || !strings.HasPrefix(code, "C") {
				code = searchLine(t, code)
			}

			code = "line:IDFM:" + code
		}

		if !strings.HasPrefix(station, "stop_area:IDFM:") {
			if _, err := strconv.ParseInt(station, 10, 64); err != nil {
				station, err = searchStation(code, station)
				if err != nil {
					c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
					return
				}
			} else {
				station = "stop_area:IDFM:" + station
			}
		}

		rurl, err := url.JoinPath(IDFM_BASEURL, "lines", code, "stops", station, "realTime")
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		requrl, err := url.Parse(rurl)
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		reqquery := url.Values{}
		reqquery.Add("MonitoringRef", station)
		requrl.RawQuery = reqquery.Encode()

		req, err := http.NewRequest("GET", requrl.String(), nil)
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		req.Header.Add("Accept", "application/json")
		req.Header.Add("apikey", IDFM_TOKEN)

		res, err := http.DefaultClient.Do(req)
		if err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}
		defer res.Body.Close()

		if res.StatusCode >= 400 {
			c.AbortWithStatusJSON(res.StatusCode, APIResult(c, nil))
			return
		}

		var schedules IDFMSchedule

		dec := json.NewDecoder(res.Body)
		if err = dec.Decode(&schedules); err != nil {
			c.AbortWithStatusJSON(http.StatusInternalServerError, gin.H{"errmsg": err.Error()})
			return
		}

		pgs := []PGSchedule{}
		for _, vehicule := range schedules.NextDepartures.Data {
			if (way == "A" && vehicule.Sens == "-1") || (way == "R" && vehicule.Sens == "1") {
				continue
			}

			msg := vehicule.Time + " mn"

			if vehicule.Code == "message" {
				msg = vehicule.Schedule
			} else if t == "rail" {
				if n, err := strconv.Atoi(vehicule.Time); err == nil {
					msg = time.Now().Add(time.Duration(n) * time.Minute).Format("15:04")
				}
			}

			pgs = append(pgs, PGSchedule{
				Destination: vehicule.LineDirection,
				Message:     msg,
				Code:        vehicule.VehicleName,
			})
		}

		c.JSON(http.StatusOK, APIResult(c, map[string][]PGSchedule{
			"schedules": pgs,
		}))
	})
}
